import assert from 'assert';
import secretShare from '../src/secretShare.js';
import utils, { constants } from '../src/utils.js';
import BigInt from 'big-integer';

const bits = 8;
const numShares = 5;
const threshold = 3;

for (let i = 0; i < 20; i++) {
    const [p, q] = utils.randomPrime(bits, 32);
    const secret = utils.random(constants.one, p.minus(constants.one));
    
    var [xs, ys] = secretShare.share(secret, numShares, threshold, p);
    var result = secretShare.interpolate(xs.slice(-threshold), ys.slice(-threshold), p)
    
    assert.equal(secret.toString(10), result.toString(10))
}

const p = BigInt('9d39f4f7fc915aded17550bccab12560edf43e03daaaf55847b91b35dcd07bb3', 16);
const original = 'In cryptography, the ElGamal encryption system is an asymmetric key encryption algorithm for public-key cryptography which is based on the Diffie–Hellman key exchange';

for (let i = 0; i < 20; i++) {
    var shares = secretShare.shareText(original, numShares, threshold, p)
    var msg = secretShare.interpolateText(shares, p)
    assert.equal(original, msg)
}